/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Marc Dreher

 *      Co-authors:
 *      ...
 */

#include "PacmanHUDinfo.h"

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "util/Convert.h"

namespace orxonox
{
    RegisterClass(PacmanHUDinfo);

    PacmanHUDinfo::PacmanHUDinfo(Context* context) : OverlayText(context)
    {
        RegisterObject(PacmanHUDinfo);

        this->PacmanGame = nullptr;
        this->bShowPoints_ = false;
        this->bShowMessage_ = false;
        this->bShowGhoststatus_ = false;
        this->messageID = 0;

    }

    void PacmanHUDinfo::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(PacmanHUDinfo, XMLPort, xmlelement, mode);

        XMLPortParam(PacmanHUDinfo, "showpoints",    setShowPoints,    getShowPoints,    xmlelement, mode).defaultValues(false);
        XMLPortParam(PacmanHUDinfo, "showmessage",   setShowMessage,   getShowMessage,   xmlelement, mode).defaultValues(false);
        XMLPortParam(PacmanHUDinfo, "showghoststatus",   setGhoststatus,   getGhoststatus,   xmlelement, mode).defaultValues(false);
        XMLPortParam(PacmanHUDinfo, "messageID",     setMessageID,     getMessageID,     xmlelement, mode).defaultValues(0);
        XMLPortParam(PacmanHUDinfo, "showgameover",   setGameover,   getGameover,   xmlelement, mode).defaultValues(false);
        XMLPortParam(PacmanHUDinfo, "showlive",   setlive,   getlive,   xmlelement, mode).defaultValues(false);
        XMLPortParam(PacmanHUDinfo, "showlevel",   setlevel,   getlevel,   xmlelement, mode).defaultValues(false);
       
    
    }

    void PacmanHUDinfo::tick(float dt)
    {
        SUPER(PacmanHUDinfo, tick, dt);

        if (this->PacmanGame)
        {
            bool death = this->PacmanGame->isdead();
            if(death && showgameover){
                const std::string& deathmessage = "Game Over";
                    setTextSize(0.08);
                    this->setCaption(deathmessage);
            }
            
            if(this->bShowPoints_) {
                    const std::string& points = "Collected points: "+multi_cast<std::string>(this->PacmanGame->getPoints()) + " of " + multi_cast<std::string>(this->PacmanGame->getTotalpoints());
                    setTextSize(0.03);
                    this->setCaption(points);
            }
  
            if(this->bShowGhoststatus_){
                    if(this->PacmanGame->getAfraid()) {
                       const std::string& ghoststatus = "Catch the ghosts! \n " + multi_cast<std::string>(this->PacmanGame->getTimer()) + " seconds left";
                       setTextSize(0.03);
                       this->setCaption(ghoststatus);
                    }
                    else{
                       const std::string& ghoststatus = "Do not get caught!"; 
                       setTextSize(0.03);
                       this->setCaption(ghoststatus);
                    }
            }

            if(this->showlive){
                const std::string& live = "Remaining lives: "+multi_cast<std::string>(this->PacmanGame->getLives());
                setTextSize(0.03);
                this->setCaption(live);
            }

            if(this->showlevel){
                const std::string& live = "Level: "+multi_cast<std::string>(this->PacmanGame->getLevel());
                setTextSize(0.03);
                this->setCaption(live);
            }


            if(death && !(this->showgameover)){
                setTextSize(0.0);
            }

        }
    }    

    void PacmanHUDinfo::changedOwner()
    {
        SUPER(PacmanHUDinfo, changedOwner);

        if (this->getOwner() && this->getOwner()->getGametype())
        {
            this->PacmanGame = orxonox_cast<Pacman*>(this->getOwner()->getGametype());
        }
        else
        {
            this->PacmanGame = nullptr;
        }
    }
}